@@ -45,7 +45,7 @@ conventions, but is very allowing for breaking them. E.g. it is
 possible to name remotes and branches with names like
 "refs/heads/master", give branches and tags the same name, or a valid
 SHA-ID as name, etc.  Similarly git allows plenty of ways to reference
-any object, in cae of ambiguities, git attempts to take the most
+any object, in case of ambiguities, git attempts to take the most
 reasonable disambiguation, and in some cases warns.
 """
 
@@ -55,30 +55,50 @@ import base64
 import sys
 from distutils.version import LooseVersion
 
-from .vcs_base import VcsClientBase
+from .vcs_base import VcsClientBase, VcsError
 
+def _get_git_version():
+    """Looks up git version by calling git --version.
+
+    :raises: VcsError if git is not installed or returns
+    something unexpected"""
+    try:
+        version = subprocess.Popen(['git --version'],
+                                   shell = True,
+                                   stdout = subprocess.PIPE).communicate()[0]
+    except:
+        raise VcsError("git not installed")
+    prefix = 'git version '
+    if version.startswith(prefix):
+        version = version[len(prefix):].strip()
+    else:
+        raise VcsError("git --version returned invalid string: '%s'"%version)
+    return version
 
 class GitClient(VcsClientBase):
     def __init__(self, path):
         """
-        Raise LookupError if git not detected
+        :raises: VcsError if git not detected
         """
         VcsClientBase.__init__(self, 'git', path)
-        with open(os.devnull, 'w') as fnull:
-            try:
-                version = subprocess.Popen(['git --version'], shell=True, stdout=subprocess.PIPE).communicate()[0]
-            except:
-                raise LookupError("git not installed, cannot create a git vcs client")
-            if version.startswith('git version '):
-                version = version[len('git version '):].strip()
-            else:
-                raise LookupError("git --version command returned invalid string: '%s'"%version)
-            self.gitversion = version
-        self.submodule_exists = self._check_git_submodules()
+        self.gitversion = _get_git_version()
 
+    @staticmethod
+    def get_environment_metadata():
+        metadict = {}
+        try:
+            version = _get_git_version()
+            resetkeep =  LooseVersion(version) >= LooseVersion('1.7.1')
+            submodules = LooseVersion(version) > LooseVersion('1.7')
+            metadict["features"] = "'reset --keep': %s, submodules: %s"%(resetkeep, submodules)
+        except VcsError as e:
+            version = "No git installed"
+        metadict["version"] = version
+        return metadict
+       
     def get_url(self):
         """
-        @return: GIT URL of the directory path (output of git info command), or None if it cannot be determined
+        :returns: GIT URL of the directory path (output of git info command), or None if it cannot be determined
         """
         if self.detect_presence():
             output = subprocess.Popen(["git config --get remote.origin.url"], shell=True, cwd=self._path, stdout=subprocess.PIPE).communicate()[0]
@@ -107,7 +127,7 @@ class GitClient(VcsClientBase):
     def update_submodules(self):
     
         # update and or init submodules too
-        if self.submodule_exists:
+        if LooseVersion(self.gitversion) > LooseVersion('1.7'):
             cmd = "git submodule update --init --recursive"
             if not subprocess.call(cmd, cwd=self._path, shell=True) == 0:
                 return False
@@ -187,12 +207,12 @@ class GitClient(VcsClientBase):
     
     def get_version(self, spec=None, fetch = True):
         """
-        @param spec: (optional) token to identify desired version. For
+        :param spec: (optional) token to identify desired version. For
         git, this may be anything accepted by git log, e.g. a tagname,
         branchname, or sha-id.
-        @param fetch: When spec is given, can be used to suppress git fetch call
+        :param fetch: When spec is given, can be used to suppress git fetch call
         
-        @return: current SHA-ID of the repository. Or if spec is
+        :returns: current SHA-ID of the repository. Or if spec is
         provided, the SHA-ID of a commit specified by some token.
         """
         if self.detect_presence():
@@ -338,10 +358,13 @@ class GitClient(VcsClientBase):
 
     def rev_list_contains(self, refname, version, fetch = True):
         """
-        calls git rev-list with refname and returns True if version can be found in rev-list result
-        @param refname a git refname
-        @param version an SHA IDs (if partial, caller is responsible for mismatch)
-        @returns
+        calls git rev-list with refname and returns True if version
+        can be found in rev-list result
+
+        :param refname: a git refname
+        :param version: an SHA IDs (if partial, caller is responsible
+        for mismatch)
+        :returns: True if version can be found in rev-list
         """
         # to avoid listing unnecessarily many rev-ids, we cut off all
         # those we are definitely not interested in
@@ -370,10 +393,11 @@ class GitClient(VcsClientBase):
         references, meaning branches or tags) for version. If it shows
         up, that means git garbage collection will not remove the
         commit. Else it would eventually be deleted.
-        @param version SHA IDs (if partial, caller is responsible for mismatch)
-        @param mask_self whether to consider direct references to this commit (rather than only references on descendants) as well
-        @param fetch whether fetch should be done first for remote refs
-        @return true if version is not recursively referenced by a branch or tag
+        
+        :param version: SHA IDs (if partial, caller is responsible for mismatch)
+        :param mask_self: whether to consider direct references to this commit (rather than only references on descendants) as well
+        :param fetch: whether fetch should be done first for remote refs
+        :returns: True if version is not recursively referenced by a branch or tag
         """
         if version != None and version != '':
             cmd = 'git show-ref -s'
@@ -395,15 +419,6 @@ class GitClient(VcsClientBase):
             return True
         return False
 
-    def _check_git_submodules(self):
-        """
-        @return: True if git version supports submodules, False otherwise,
-        including if version cannot be detected
-        """
-        # 'git version 1.7.0.4\n'
-        return LooseVersion(self.gitversion) > LooseVersion('1.7')
-
-
     def _do_fetch(self):
         if not subprocess.call("git fetch", cwd=self._path, shell=True) == 0:
             return False
@@ -449,4 +464,5 @@ class GitClient(VcsClientBase):
             return False
         return True
 
+    
 GITClient=GitClient
