@@ -33,36 +33,65 @@
 """
 tar vcs support.
 
-New in ROS C-Turtle.
 """
 
 import subprocess
 import os
-import yaml
 import urllib
 import tempfile
 import sys
 import shutil
 
-from .vcs_base import VcsClientBase
+_yaml_missing = False
+try:
+    import yaml
+except:
+    _yaml_missing = True
+
+from .vcs_base import VcsClientBase, VcsError
+
+def _get_tar_version():
+    """Looks up tar version by calling tar --version.
+
+    :raises: VcsError if git is not installed or returns
+    something unexpected"""
+    try:
+        version = subprocess.Popen(['tar --version'],
+                                   shell = True,
+                                   stdout = subprocess.PIPE).communicate()[0]
+    except:
+        raise VcsError("git not installed")
+    if version.startswith('tar '):
+        version = version.splitlines()[0][len('tar '):].strip()
+    else:
+        raise VcsError("tar --version returned invalid string: '%s'"%version)
+    return version
 
 class TarClient(VcsClientBase):
 
     def __init__(self, path):
         """
-        @raise KeyError if tar not detected
+        @raise VcsError if tar not detected
         """
         VcsClientBase.__init__(self, 'tar', path)
         self.metadata_path = os.path.join(self._path, ".tar")
-        with open(os.devnull, 'w') as fnull:
-            try:
-                subprocess.call("tar --help".split(), stdout=fnull, stderr=fnull)
-            except:
-                raise LookupError("tar not installed, cannnot create an tar vcs client")
- 
+        _get_tar_version()
+        if _yaml_missing:
+            raise VcsError("Python yaml libs could not be imported. Please install python-yaml. On debian systems sudo apt-get install python-yaml")
+
+    @staticmethod
+    def get_environment_metadata():
+        metadict = {}
+        try:
+            version = _get_tar_version()
+        except VcsError as e:
+            version = "No tar installed"
+        metadict["version"] = version
+        return metadict
+
     def get_url(self):
         """
-        @return: TAR URL of the directory path (output of tar info command), or None if it cannot be determined
+        :returns: TAR URL of the directory path (output of tar info command), or None if it cannot be determined
         """
         if self.detect_presence():
             with open(self.metadata_path, 'r') as metadata_file:
@@ -97,14 +126,15 @@ class TarClient(VcsClientBase):
                 metadata = yaml.dump({'url': url, 'version':version})
                 with open(self.metadata_path, 'w') as md:
                     md.write(metadata)
-
+                if os.path.exists(tempdir):
+                    shutil.rmtree(tempdir)
                 return True
             else:
-                print "failed to extract"
-                os.shutil.rmtree(filename)
-                return False
-        except:
-            sys.stderr.write("Tarball download unpack failed\n")
+                sys.stderr.write("failed to extract")
+            
+            shutil.rmtree(tempdir)
+        except Exception as e:
+            sys.stderr.write("Tarball download unpack failed%s\n"%str(e))
             return False
         return False
 
